=head1 DESCRIPTION

Perl module to read waypoints from a text file.

=head1 AUTHOR

Copyright (c) 2003 Scott Haywood.  All rights reserved.

This package is free software and is provided "as is" without express
or implied warranty.  It may be used, redistributed and/or modified
under the terms of the Perl Artistic License (see
http://www.perl.com/perl/misc/Artistic.html)

=head1 SYNOPSIS

use p_waypoints;
$w = p_waypoints->new("waypoints.txt");

@list = $w->list;
for my $p (@list) {
	printf("name:%s, coords:%s,%s\n", $$p{name},$$p{lat},$$p{long});
}

=head1 SAMPLE WAYPOINT FILE

...
WP,D,M1 , 28.19874, -177.32785,...
WP,D,M2 , 28.19111, -177.32373,...
WP,D,M3 , 28.18947, -177.34239,...
WP,D,X0 , 28.25000, -177.33333,...
WP,D,X1 , 28.25000, -177.43333,...
WP,D,X2 , 28.21667, -177.40000,...
...

=cut

package p_waypoints;
use MyAutoLoader2;
use strict;

use vars qw(@ISA $a $b);
@ISA = qw(MyAutoLoader2);


sub new {
	my $that  = shift; # class name or object reference
	my $class = ref($that) || $that;

	my $filename = shift or die "no file name";

	my $input;
	unless (open($input, $filename)) {
	   print STDERR "Can't open $filename: $!\n";
	   return;
	}
	my @wp = ();
	LINE: while (<$input>) {
		#print $_;
		next LINE unless /^WP/;
		#print "waypoint:", $_;
		my ($wp,$d, $name, $lat, $long) = split /\s?,\s?/;
		chomp $long;
		push @wp, {wp=>$wp, d=>$d, name=>$name, lat=>$lat, long=>$long};
	}
	## my @sorted_wp = sort { $$a{name} cmp $$b{name} } @wp;
	my %permitted = (filename=>$filename,
	                 list=>\@wp
					 );
	my $x = $class->SUPER::new(\%permitted, @_);

	return $x;
}

1;
__END__
