##
##

package Debug;

use Carp;
use strict;

require Exporter;
use vars qw(@ISA @EXPORT_OK);
@ISA = qw(Exporter);
@EXPORT_OK = qw(&say &say2 &sayN &class_debug &debug);

# Private variables
#my %table = ();
use vars qw(%table);
%table = ();

#########################################################
# methods
#########################################################

# Accesses the debug flag for the given class
#
sub class_debug {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    if (@_ > 1) {
    	confess "class_debug: too many args";
    }
	if (@_ == 1) {
    	$table{$class} = shift;
    } elsif ( !defined( $table{$class} )) {
		$table{$class} = 0;
	}
    return $table{$class};
}

# Accesses the debug flag for the caller's class.
# Usage: &debug(val) or &debug()
# Don't call as $obj->debug(); this will not have
# the expected results; use $obj->class_debug() instead.
#
sub debug {
	my $caller = caller;
	return class_debug($caller, @_);
}

sub _sayN {
	my $lev = shift;
	my $caller = shift;
	$caller = __PACKAGE__ if (!$caller);
	if (class_debug($caller) >= $lev) {
		print STDERR $caller,": ",@_, "\n";
	}
}

sub say {
	_sayN (1, scalar caller, @_, );
}

sub say2 {
	_sayN (2, scalar caller, @_);
}

sub sayN {
	_sayN (shift, scalar caller, @_);
}

1;
__END__

=head1 NAME

Debug - utility class for debugging

=head1 SYNOPSIS

package mypackage;
use Debug qw(&say &say2 &debug);

sub foo {
	my $self = shift;
	my $class = ref($self) || $self;
	say 'enter foo ';
	debug(1);
	say '1. this should print';
	debug(0);
	say '(this should not print)';
	debug(2);
	say2 '2. this should also print';
}
my $m = {};
bless $m;
$m->foo;

1;


=head1 AUTHOR

Copyright (c) 2003 Scott Haywood.  All rights reserved.

This package is free software and is provided "as is" without express
or implied warranty.  It may be used, redistributed and/or modified
under the terms of the Perl Artistic License (see
http://www.perl.com/perl/misc/Artistic.html)

=cut
