
package MyAutoLoader2;

use strict;
use Debug qw(&debug &class_debug &say &say2);
use vars qw(@ISA);
#@ISA = ("Debug");

# allow Carp to be used
#
#use Carp qw(croak carp);
use Carp;

# allow "autoloaded" methods
# (see perlsub).
#
use vars qw($AUTOLOAD);  # it's a package global

###################################
# Internal methods
###################################

#
my $pkg = __PACKAGE__;

# initialize the list of namespaces
#
sub _init_blocks() {
	my $self = shift;
	my $class = ref($self)
		or croak "$self is not an object";
	$$self{$pkg} = [];
}

# get a reference to the array of namespaces
#
sub _get_blocks() {
	my $self = shift;
	my $class = ref($self)
		or croak "$self is not an object";
	return $$self{$pkg};
}

# add a namespace
#
sub _push_block() {
	my $self = shift;
	my $class = ref($self)
		or croak "$self is not an object";
	my $fields = shift;
	croak "not a hash: '$fields'" if ref($fields) ne 'HASH';
	unshift @{ $$self{$pkg} }, $fields;
}

# Adds a block (namespace) to the front of the list for this object.
# $_[0] should be a hash of field names and default values;
# returns 0 otherwise.
# Remaining args are fields to be set.
# Returns 0 if any of those remaining fields are not in the hash;
# otherwise returns a true value.
#
sub _add_block() {
	my $self = shift;
	my $class = ref($self)
		or croak "$self is not an object";

	my $auto_fields = shift;
	my $fields = {};
	if (ref($auto_fields) eq 'HASH') {
    	$fields = $auto_fields;
	} else {
		return 0; # -------------->
	}
	
	# Add the fields given as arguments (any could be undef)
	#
	while (@_) {
		my $fld = shift;
		my $val = shift;
		if (exists $$fields{$fld}) {
			return 0 if ref($fld); # --------->
			$$fields{$fld} = $val
		} else {
			say "not a valid field '", $fld, "'";
			return 0; # ---------->
		}
	}

	# The object will be a hash, with one value;
	# the value is a list of blocks (namespaces);
	# each block is a hash of field names and values.
	# $self is { key => [ {name1=>val1,...} ] }
	#
	$self->_push_block( $fields );

	return 1;
}

# Constructor
# Called using ->new;
#
sub _new {

    my $that  = shift; # class name or object reference
	my $class = ref($that) || $that;
	
	say ' _new: $class=', $class;

	my $self = {};
	bless $self, $class; # make it an object of the desired type

	$self->_init_blocks;
	if ( !$self->autoload(@_) ) {
		say "_new: bad args";
		return undef; # ------------->
	}

	say '_new: returning ', $self ;
    return $self;
}

##########################################
# methods
##########################################

# Add a namespace to the object
#
sub autoload {

	my $self = shift;
	my $class = ref($self)
		or croak "$self is not an object";
	
	say 'autoload: $class=', $class;
	my $rv;
	$rv = (ref($_[0]) eq 'HASH')
	      ? $self->_add_block(@_)
		  : $self->_add_block({ @_ });
	return $rv;
}


# Return a string version of the object
#
sub to_string {
    my $self = shift;
    croak "$self is not an object" unless ref($self);
    my $type = ref($self);
    my $rv = $type . '{';
#    for my $k (keys %$self) {
#		say  'to_string :', $k, '=',
#    		 (defined($$self{$k}) ? $$self{$k} . ':' : "(undefined)");
#    	if (defined $$self{$k}) {
#    		$rv .= $k . '=' . $$self{$k} . ','
#    			unless substr($k,0,1) eq '_';  # ignore fields beginning with '_'
#        }
#    }
	my $blocks = $self->_get_blocks;
	for my $f (@$blocks) {
		$rv .= '{';
		for my $k (keys %$f) {
			my $v = $$f{$k} || '';
			$rv .= $k . '=' . $v . ',';
		}
		chop $rv;
		$rv .= "}";
	}
    $rv .= '}';
    return $rv;
}

# object destructor
sub _destroy {
	my $thing = shift;
	say "_destroy, thing=$thing";
}

my %indirect = (
	new => \&_new,
	DESTROY => \&_destroy,
);

sub _get_namespace {
    my $self = shift;
    my $type = ref($self) || $self;

    my $fieldname = shift
    	or croak "no field name";
    croak "$self is not an object"
    	unless ref($self);

	my $namespace;
	my $blocks = $self->_get_blocks;
	for my $f (@$blocks) {
		if (exists $$f{$fieldname}) {
			$namespace = $f;
			last;
		}
	}

    croak "Can't access `$fieldname' field in class $type"
    	unless defined($namespace);
	return $namespace;
}

# Get a field's value.
# This returns a scalar.
#
# If the field is an array or hash, a reference is returned.
# (For an array, this differs from using the fieldname directly ($obj->fieldname),
# where the array contents would be returned.  See the AUTOLOAD method)
#
sub getfield {
    my $self = shift;
    my $type = ref($self) || $self;

    my $fieldname = shift
    	or croak "no field name";
	my $ns = $self->_get_namespace($fieldname);
	return $ns->{$fieldname};
}	


sub AUTOLOAD {
    my $self = shift;
    my $type = ref($self) || $self;

	#debug(1);

    my $name = $AUTOLOAD;
    $name =~ s/.*://;   # strip fully-qualified portion
	say "AUTOLOAD: method is $name";
	say "AUTOLOAD: # args = ", scalar @_;
	
	# Make an indirect method call if possible
	#
	my $meth = $indirect{$name};
	if (ref($meth) eq 'CODE') {
		say "AUTOLOAD: substitute method is $meth" ;
		return &$meth($self, @_);	
	}

	# Implement the method here if possible
	#
    croak "$self is not an object"
    	unless ref($self);

	my $namespace = $self->_get_namespace($name);
	my $key = $name; # the hash key

	# set the value if given
	#
    if    (@_ == 1) {
		$namespace->{$key} = shift;
	}
    elsif (@_ >  1) {
		$namespace->{$key} = [ @_ ];
	}

	# Return the value.
	# For an array, return its contents (or, in a scalar context, its size).
	#
	if (ref($namespace->{$key}) eq 'ARRAY') {
		return  @{ $namespace->{$key} }
	}
	else {
		return $namespace->{$key};
	}
}

1;
__END__

=head1 NAME

MyAutoLoader2 - implements an AUTOLOAD method

=head1 SYNOPSIS

 package test;
 use MyAutoLoader2;
 use vars qw(@ISA);
 @ISA = qw(MyAutoLoader2);
 
 # create an object with 2 fields
 $x = __PACKAGE__->SUPER::new(foo=>1, bar=>2);
 
 print $x->foo, "\n";       # print the value of a field
 $x->foo(11);               # set a field
 $x->autoload(another=>3);  # add another field
 print $x->another, "\n";   # print the new field's value
 print $x->to_string, "\n"; # print the object

=head1 EXAMPLE

 package test;
 use MyAutoLoader2;
 use strict;
 
 use vars qw(@ISA);
 @ISA = qw(MyAutoLoader2);
 
 # create an object in a derived class
 sub new {
 	my $that  = shift; # class name or object reference
 	my $class = ref($that) || $that;
 
 	my %permitted = (foo=>1, bar=>2);
 	my $x = $class->SUPER::new(\%permitted, @_);
 	return $x;
 }
 
 {
 package t2;
 use vars qw(@ISA);
 @ISA = qw(test);
 my $c = 't2';
 
 # create an object of a class derived from a derived class
 my $x = $c->SUPER::new;
 my $y = $c->SUPER::new(foo=>888);
 my $z = $c->SUPER::new(notafield=>777);
 print $x->to_string, "\n";
 print $y->to_string, "\n";
 print q{$z is }, (!defined($z) ? "undefined" : $z),"\n";
 
 print "foo=",$x->foo, "\n";
 print "bar=",$x->bar, "\n";
 
 # add another set of fields; some may override previous.
 $x->autoload(bar=>999, xxx=>3, yyy=>4, zzz=>undef, aaa=>undef);
 print "foo=",$x->foo, "\n";
 print "bar=",$x->bar, "\n";
 print "xxx=",$x->xxx, "\n";
 
 # print the object
 print $x->to_string, "\n";
 }

=head1 AUTHOR

Copyright (c) 2003 Scott Haywood.  All rights reserved.

This package is free software and is provided "as is" without express
or implied warranty.  It may be used, redistributed and/or modified
under the terms of the Perl Artistic License (see
http://www.perl.com/perl/misc/Artistic.html)

=cut
